#!/bin/bash
#
# Script for running ScaFES applications.
#
# Copyright (c) 2011-2013, ZIH, TU Dresden, Federal Republic of Germany.
# For details, see the files COPYING and LICENSE in the base directory
# of the package.

set -u

#------------------------------------------------------------------------------#
declare -i local spaceDim=$SCAFES_SPACE_DIM;
declare -i local nTestsMpi=$SCAFES_N_TESTS_MPI;
declare -i local nTestsOpenMp=$SCAFES_N_TESTS_OPENMP;
declare    local testForWeakScaling=$SCAFES_TEST_FOR_WEAK_SCALING;
declare -i local nProcessesMpiStart=$SCAFES_N_PROCESSES_MPI_START;
declare -i local nThreadsOpenMpStart=$SCAFES_N_THREADS_OPENMP_START;
declare -i local nTimesteps=$SCAFES_N_TIMESTEPS;
declare -i local nSnapshots=$SCAFES_N_SNAPSHOTS;
declare -i local starttime=$SCAFES_START_TIME;
declare -i local endtime=$SCAFES_END_TIME;
declare    local nameConfigfile=$SCAFES_NAME_CONFIGFILE;
declare    local nameKindfile=$SCAFES_NAME_KINDFILE;
declare    local debugApplication=$SCAFES_DEBUG_APPLICATION;
declare    local outputlevel=$SCAFES_OUTPUT_LEVEL;
declare -i local eins=1;

#------------------------------------------------------------------------------#
# Read in name of the problem.
declare    local nameProblem=$1

#------------------------------------------------------------------------------#
declare -i local nProblems;
nameProblemVect=("$nameProblem");
nProblems=1;
#------------------------------------------------------------------------------#
if [ "$debugApplication" == "yes" ] ; then
    nTestsMpi=1;
    nProcessesMpiStart=1;
    nThreadsOpenMpStart=1;
fi
#------------------------------------------------------------------------------#
if [ "$nameKindfile" == "" ] ; then
    optionKindfile="";
else
    optionKindfile="--kindfile=$nameKindfile";
fi
#------------------------------------------------------------------------------#
if [ "$nameConfigfile" == "" ] ; then
    optionConfigfile="";
else
    optionConfigfile="--configfile=$nameConfigfile";
fi

declare -i local endTestMpi;
declare -i local endTestOpenMp;
declare -i local endSpacedim;
declare    local currNameProblem;
declare -i local endProblem;
declare -i local currIdxProblem;
declare -i local currTest;
declare -i local currIdTest;
declare -i local currIdTestMpi;
declare -i local currIdTestOpenMp;
declare -i local currNprocessesMpi;
declare -i local currNthreadsOpenMp;
declare -i local currEndProblem;
declare -i local currEndTest;
declare -i local currNnodesTotal;
declare    local currNameExecutable; # Relative path to executable
declare -i tmpNodesDir;
endTestMpi=${nTestsMpi}-${eins};
endTestOpenMp=${nTestsOpenMp}-${eins};
endProblem=${nProblems}-${eins};
endSpacedim=${spaceDim}-${eins};
currEndTest=${nTestsMpi}*${nTestsOpenMp};
currEndProblem=${nProblems};

#------------------------------------------------------------------------------#
# Perform a loop over all problems.
for idxProblem in `seq 0 $endProblem`; do
    currNameProblem=${nameProblemVect[idxProblem]};
    currIdxProblem=${idxProblem}+${eins};
    currNameExecutable="./${currNameProblem}";
    currIdTest=$eins;

    #--------------------------------------------------------------------------#
    # Perform a loop over all MPI tests.
    currNprocessesMpi=${nProcessesMpiStart};
    for idxTestMpi in `seq 0 $endTestMpi`; do
        currIdTestMpi=${idxTestMpi}+${eins};

        #----------------------------------------------------------------------#
        # Perform a loop over all OpenMP tests.
        currNthreadsOpenMp=${nThreadsOpenMpStart};
        for idxTestOpenMp in `seq 0 $endTestOpenMp`; do
            currIdTestOpenMp=${idxTestOpenMp}+${eins};
            #currNnodes=(${SCAFES_N_NODES[*]})
            currStarttime=${starttime};
            currEndtime=${endtime};
            currNtimesteps=${nTimesteps};
            currNsnapshots=${nSnapshots};
            currNnodesTotal=1;
            for idxDim in `seq 0 $endSpacedim`; do
                currNnodesTotal=${currNnodesTotal}*${SCAFES_N_NODES[${idxDim}]};
            done

            echo -e "\n\n  +-------------------------------+";
            echo -e "  | PROBLEM: ${currIdxProblem} / ${currEndProblem}";
            printf "  |     * Name of Problem   = %s\n" ${currNameProblem};
            echo -e "  | TEST: ${currIdTest} / ${currEndTest}";
            printf "  |     * #{MPI processes}  = %s\n" ${currNprocessesMpi};
            printf "  |     * #{OpenMP threads} = %s\n" ${currNthreadsOpenMp};
            printf "  |     * #{Nodes total}    = %s\n" ${currNnodesTotal};
            for idxDim in `seq 0 $endSpacedim`; do
                printf "  |     * #{Nodes dim. $idxDim}   = %s\n" ${SCAFES_N_NODES[${idxDim}]};
            done
            printf "  |     * #{Start time}     = %s\n" ${currStarttime};
            printf "  |     * #{End time}       = %s\n" ${currEndtime};
            printf "  |     * #{Time steps}     = %s\n" ${currNtimesteps};
            printf "  |     * #{Snap shots}     = %s\n" ${currNsnapshots};
            echo -e "  +-------------------------------+\n\n";

            #------------------------------------------------------------------#
            divideGrid=${SCAFES_PARTITION_GRID[0]};
            currNnodes="${SCAFES_N_NODES[0]}";
            currCoordNodeFirst="${SCAFES_COORD_NODE_FIRST[0]}";
            currCoordNodeLast="${SCAFES_COORD_NODE_LAST[0]}";
            for idxDim in `seq 1 $endSpacedim`; do
                divideGrid="${divideGrid}x${SCAFES_PARTITION_GRID[${idxDim}]}";
                currNnodes="${currNnodes}x${SCAFES_N_NODES[${idxDim}]}";
                currCoordNodeFirst="${currCoordNodeFirst}x${SCAFES_COORD_NODE_FIRST[${idxDim}]}";
                currCoordNodeLast="${currCoordNodeLast}x${SCAFES_COORD_NODE_LAST[${idxDim}]}";
            done
            namePartfile="${currNameProblem}_${currNprocessesMpi}_${currNthreadsOpenMp}_${currNnodes}.partfile"
            nameOutputfile="${currNameProblem}_${currNprocessesMpi}_${currNthreadsOpenMp}"
            #nameOutputfile="${currNameProblem}_${currNprocessesMpi}_${currNnodes}.out"

            #------------------------------------------------------------------#
            echo "--- Run application. ---"
            if [ "$debugApplication" == "yes" ] ; then
            (
                set -x
                export OMP_NUM_THREADS=$currNthreadsOpenMp;
                libtool --mode=execute gdb --args \
                    $currNameExecutable \
                    --dim=$spaceDim \
                    --nPartitions=$currNprocessesMpi \
                    --divideGrid=$divideGrid \
                    --nNodes=$currNnodes \
                    --coordNodeFirst=$currCoordNodeFirst \
                    --coordNodeLast=$currCoordNodeLast \
                    --starttime=$currStarttime \
                    --endtime=$currEndtime \
                    --nTimesteps=$currNtimesteps \
                    --nSnapshots=$currNsnapshots \
                    ${optionKindfile} \
                    ${optionConfigfile} \
                    --writePartitionfile=$namePartfile \
                    --outputfile=$nameOutputfile \
                    --outputlevel=$outputlevel \
            )
            else
            (
                # -npernode 64
                set -x
                export OMP_NUM_THREADS=$currNthreadsOpenMp;
                mpirun -np $currNprocessesMpi \
                    $currNameExecutable \
                    --dim=$spaceDim \
                    --nPartitions=$currNprocessesMpi \
                    --divideGrid=$divideGrid \
                    --nNodes=$currNnodes \
                    --coordNodeFirst=$currCoordNodeFirst \
                    --coordNodeLast=$currCoordNodeLast \
                    --starttime=$currStarttime \
                    --endtime=$currEndtime \
                    --nTimesteps=$currNtimesteps \
                    --nSnapshots=$currNsnapshots \
                    ${optionKindfile} \
                    ${optionConfigfile} \
                    --writePartitionfile=$namePartfile \
                    --outputfile=$nameOutputfile \
                    --outputlevel=$outputlevel
            )
            fi

            currNthreadsOpenMp=$currNthreadsOpenMp*2;
            currIdTest=$currIdTest+$eins;
        done

        #------------------------------------------------------------------#
        # Test for weak scalability:
        # Problem size increases linearly with number of processes.
        if [ "$testForWeakScaling" == "yes" ]; then
            for idxDim in `seq 0 $endSpacedim`; do
                if [ ${SCAFES_PARTITION_GRID[$idxDim]} -eq 1 ]; then
                    tmpNodesDir=${SCAFES_N_NODES[${idxDim}]}*2;
                    SCAFES_N_NODES[${idxDim}]=$tmpNodesDir;
                    currNprocessesMpi=$currNprocessesMpi*2;
                fi
            done
        #------------------------------------------------------------------#
        # Test for strong scalability:
        # Problem size remains constant.
        # The number of processes increases linearly.
        else
            currNprocessesMpi=$currNprocessesMpi*2;
        fi
    done
done
